# aggregateMonthly.py aggregates *month_[0-9]*csv by raptor name on cmd line
# It sticks a garbage row of all blanks as the last row of each output file
# for testing. D. Parson, Fall 2023, CSC223 & CSC523.
import sys
import csv

aggregates = {}

if __name__ == '__main__':  # called from command line
    for csvname in sys.argv[1:]:    # sys.argv[0] is 'aggregateMonthly.py'
        if (not csvname.endswith('.csv')):
            raise ValueError('INVALID CSV FILE NAME: ' + csvname)
        f = open(csvname, 'r')
        csvf = csv.reader(f)
        hdr = csvf.__next__()
        if (len(hdr) != 29) or (not hdr[-1].endswith('_All')):
            raise ValueError('INVALID COLUMN COUNT OR TARGET NAME ON HEADER '
                + 'FOR FILE ' + csvname + ', COUNT ' + str(len(hdr))
                + ' != 29 OR TARGET DOES NOT END IN _All: ' + hdr[-1])
        underline = hdr[-1].index('_')  # raptor name comes before this
        raptor = hdr[-1][0:underline]   # slice out AK, BW, etc.
        if not raptor in aggregates.keys():
            lastunderline = csvname.rindex('_') # e.g., NH_month_9.csv
            outname = csvname[0:lastunderline] + '.csv'
            outfile = open(outname, 'w')
            outcsv = csv.writer(outfile)
            lenrow = len(hdr)
            aggregates[raptor] = (outname, outfile, outcsv, len(hdr))
            outcsv.writerow(hdr)    # Write hdr only first time seen.
        else:
            outname = aggregates[raptor][0]
            outcsv = aggregates[raptor][2]
            lenrow = aggregates[raptor][3]
        print('Aggregating ',csvname,'into',outname)
        for row in csvf:
            if len(row) != lenrow:
                sys.stderr.write('SKIP INVALID ROW LEN '
                    + str(len(row)) + ': ' + str(row) + '\n')
            else:
                outcsv.writerow(row)    # minimize in-core data
        # Next file may be same raptor or may be different.
    for raptor in aggregates.keys():
        garbagerow = ['' for i in range(0,lenrow)]
        outcsv = aggregates[raptor][2]
        outcsv.writerow(garbagerow)
        aggregates[raptor][1].close()
