# loadAndStoreArff.py loads file given on command lines and saves using
# my custom arfflib_3_3, D. Parson Fall 2023 CSCx23

import sys
import csv      # no csv.writer in arfflib, there is a writeARFF
from arfflib_3_3 import readCSV, Normalize, projectARFF, joinARFF, writeARFF
from pprint import pformat

if __name__ == '__main__':  # run from command line
    attrmap, dataset, CSVheaderRow, nameToCol, colToName, colToType =   \
        readCSV(sys.argv[1])
    # read whole dataset into memory since that is what np.genfromtxt does
    print('Type info:',attrmap)
    print('First row: ', pformat(list(dataset[0]),width=80))
    print('Last row: ', pformat(list(dataset[-1]),width=80))

    # Database select:
    yearcol = nameToCol['year']
    monthcol = nameToCol['month']
    def filtFunc(yearcol,monthcol):
        # This is a closure that binds yearcol,monthcol in the outer function.
        def f(row):
            try:
                return row[yearcol] == 1994 and row[monthcol] == 10
            except:
                return False
        return f
    print()
    for row in filter(filtFunc(yearcol,monthcol), dataset):
        print('SELECTION',row)
    print('')

    # Database project of columns [0], [1], [-1] (last)
    projection = [(row[yearcol], row[monthcol], row[-1]) for row in dataset]
    # They are in random order. Sort by year, month
    # NO! PRESERVE ROW ORDER UNTIL JOIN. sorted() is not a mutator.
    # projection.sort(key=lambda row : (row[0], row[1]))
    def sortFunc(row):
        try:
            pair = (float(row[0]), float(row[1]))
        except:
            pair = (-1, -1) # sort undefined year or month to first
            # Using float is OK. We are not keeping these sort keys in data.
        return pair
    sortedTriplet = sorted(projection, key=sortFunc)
    print('PROJECTION of [0, 1, -1]:\n',pformat(projection), '\n')

    # Database join of two projections.
    normNames = ['HMtempC_mean', 'WindSpd_mean', 'wnd_WNW_NW']
    moreNames = normNames + [CSVheaderRow[-1]]
    normattrs, normdata = projectARFF(attrmap, dataset,
        moreNames, isKeepingAttributes=True)
    # I am putting target as last attribute for Weka.
    normattrs, normdata = Normalize(normattrs, normdata, normNames)
    timeattrs, timedata = projectARFF(attrmap, dataset,
        ['year', 'month'], isKeepingAttributes=True)
    joinedAttrs, joinedDataset = joinARFF(timeattrs, timedata,
        [(attrname, 'numeric') for attrname in moreNames],
        normdata)

    print('JOIN NORMALIZED: ', joinedAttrs)
    joinedDataset.sort(key=sortFunc)
    print(pformat(joinedDataset), '\n')

    extensionIndex = sys.argv[1].find('.csv')
    outname = sys.argv[1][0:extensionIndex] + '_Arff' + '.csv'
    # np.savetxt(outname, dataset, delimiter=',', newline='\n')
    # ^^^ THAT LINES LOSES HEADERS, SAVES THEM ALL AS FLOATS
    ofile = open(outname, 'w')
    ocsv = csv.writer(ofile)
    ocsv.writerow(CSVheaderRow)
    ocsv.writerows(dataset)
    ofile.close()
    writeARFF(outname.replace('.csv','.arff'), 'demo_arfflib', joinedAttrs,
        joinedDataset, clobber=True)
