# loadAndStorePython.py loads file given on command lines and saves using csv
# D. Parson Fall 2023 CSCx23

import sys
import csv
from pprint import pformat

if __name__ == '__main__':  # run from command line
    inputfile = open(sys.argv[1], 'r')
    inputcsv = csv.reader(inputfile)
    header = inputcsv.__next__()
    def makenum(thing): # csv inputs every cell as a string
        result = None
        try:
            result = int(thing) # might fail on decimal point
        except:
            try:
                result = float(thing)
            except:
                result = thing.strip()  # empty string
        return result
    dataset = []
    for row in inputcsv:
        for cellix in range(0, len(row)):
            row[cellix] = makenum(row[cellix]) # This might mix ints and floats
        dataset.append(row)
    inputfile.close()
    # read whole dataset into memory since that is what np.genfromtxt does
    print('First row: ', pformat(list(dataset[0]),width=80))
    print('Last row: ', pformat(list(dataset[-1]),width=80))

    # Database select:
    yearcol = header.index('year')
    monthcol = header.index('month')
    def filtFunc(yearcol,monthcol):
        # This is a closure that binds yearcol,monthcol in the outer function.
        def f(row):
            try:
                return row[yearcol] == 1994 and row[monthcol] == 10
            except:
                return False
        return f
    print()
    for row in filter(filtFunc(yearcol,monthcol), dataset):
        print('SELECTION',row)
    print('')

    # Database project of columns [0], [1], [-1] (last)
    projection = [(row[0], row[1], row[-1]) for row in dataset]
    # They are in random order. Sort by year, month
    # NO! PRESERVE ROW ORDER UNTIL JOIN. sorted() is not a mutator.
    # projection.sort(key=lambda row : (row[0], row[1]))
    def sortFunc(row):
        try:
            pair = (float(row[0]), float(row[1]))
        except:
            pair = (-1, -1) # sort undefined year or month to first
            # Using float is OK. We are not keeping these sort keys in data.
        return pair
    sortedTriplet = sorted(projection, key=sortFunc)
    print('PROJECTION of [0, 1, -1]:\n',pformat(projection), '\n')

    # Database join of two projections.
    HMtempC_mean_col = header.index('HMtempC_mean')
    WindSpd_mean_col = header.index('WindSpd_mean')
    wnd_WNW_NW_col = header.index('wnd_WNW_NW')
    # Find min and max per columns of interest for [0.0, 1.0] normalization.
    minmax = [[dataset[1][HMtempC_mean_col], dataset[1][HMtempC_mean_col]],
        [dataset[1][WindSpd_mean_col], dataset[1][WindSpd_mean_col]],
        [dataset[1][wnd_WNW_NW_col], dataset[1][wnd_WNW_NW_col]]]
    for row in dataset:
        try:
            minmax[0][0] = min(minmax[0][0], row[HMtempC_mean_col])
            minmax[0][1] = max(minmax[0][1], row[HMtempC_mean_col])
        except:
            pass    # Cannot update empty field
        try:
            minmax[1][0] = min(minmax[1][0], row[WindSpd_mean_col])
            minmax[1][1] = max(minmax[1][1], row[WindSpd_mean_col])
        except:
            pass    # Cannot update empty field
        try:
            minmax[2][0] = min(minmax[2][0], row[wnd_WNW_NW_col])
            minmax[2][1] = max(minmax[2][1], row[wnd_WNW_NW_col])
        except:
            pass    # Cannot update empty field
    joinedDataset = []
    for rowindex in range(0, len(dataset)):
        try:
            # scale each value the percentage of the way from its min to max
            norm_HMtempC_mean_col = round(((dataset[rowindex][HMtempC_mean_col]
                - minmax[0][0]) / (minmax[0][1] - minmax[0][0])),6)
        except:
            norm_HMtempC_mean_col = ''
        try:
            norm_WindSpd_mean_col = round(((dataset[rowindex][WindSpd_mean_col]
                - minmax[1][0]) / (minmax[1][1] - minmax[1][0])),6)
        except:
            norm_WindSpd_mean_col = ''
        try:
            norm_wnd_WNW_NW_col = round(((dataset[rowindex][wnd_WNW_NW_col]
                - minmax[2][0]) / (minmax[2][1] - minmax[2][0])),6)
        except:
            norm_wnd_WNW_NW_col = ''
        joinedDataset.append(projection[rowindex]
            + (norm_HMtempC_mean_col, norm_WindSpd_mean_col,
                norm_wnd_WNW_NW_col))
    jhdr = (header[0], header[1], header[-1],
        header[HMtempC_mean_col], header[WindSpd_mean_col],
        header[wnd_WNW_NW_col])
    print('JOIN NORMALIZED: ', jhdr)
    joinedDataset.sort(key=sortFunc)
    print(pformat(joinedDataset), '\n')


    extensionIndex = sys.argv[1].find('.csv')
    outname = sys.argv[1][0:extensionIndex] + '_Python' + '.csv'
    # np.savetxt(outname, dataset, delimiter=',', newline='\n')
    # ^^^ THAT LINES LOSES HEADERS, SAVES THEM ALL AS FLOATS
    ofile = open(outname, 'w')
    ocsv = csv.writer(ofile)
    ocsv.writerow(header)
    ocsv.writerows(dataset)
    ofile.close()
